/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


void CreateRadioMenus()
{
	CreateRadioBaseMenu();
	
	if (g_iRadioPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL || g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST)
		CreateRadioVolumeMenu();
	
	if (g_bPlayListFound)
	    CreateGenreAndStationMenus();
	
	// ShoutCast custom stations menu initialization
	if (g_bShoutCastCustom && g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST && g_bPlayListFound) {
		CreateCustomGenreAndStationMenus();
	}
}

void CreateRadioBaseMenu()
{
	char szTranslation[64] = "";
	
	g_mRadioBaseMenu = new Menu(Handler_BaseRadioMenu);
	g_mRadioBaseMenu.SetTitle("SourceMod Radio:");
	
	if (g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST && g_bShoutCastCustom)
	{
		Format(szTranslation, sizeof(szTranslation), "%T", "ShoutCast Stations", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("1", szTranslation);
		Format(szTranslation, sizeof(szTranslation), "%T", "Server Custom Stations", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("2", szTranslation);
		Format(szTranslation, sizeof(szTranslation), "%T", "Volume", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("3", szTranslation);
		Format(szTranslation, sizeof(szTranslation), "%T", "Turn Off", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("4", szTranslation);
	}
	else if (g_bUseGenres)
	{
		Format(szTranslation, sizeof(szTranslation), "%T", "Select Genre", LANG_SERVER);
		
		g_mRadioBaseMenu.AddItem("1", szTranslation);
		
		// Check if we need to add volume control to the base menu
		if (g_iRadioPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL || g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST)
		{
			Format(szTranslation, sizeof(szTranslation), "%T", "Volume", LANG_SERVER);
			g_mRadioBaseMenu.AddItem("2", szTranslation);
		}
		
		Format(szTranslation, sizeof(szTranslation), "%T", "Turn Off", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("3", szTranslation);
	}
	else
	{
		Format(szTranslation, sizeof(szTranslation), "%T", "Select Station", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("1", szTranslation);
		if (g_cVarRadioPlayType.IntValue >= RADIO_PLAY_TYPE_VOLUME_CONTROL)
		{
			Format(szTranslation, sizeof(szTranslation), "%T", "Volume", LANG_SERVER);
			g_mRadioBaseMenu.AddItem("2", szTranslation);
		}
		Format(szTranslation, sizeof(szTranslation), "%T", "Turn Off", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("3", szTranslation);
	}
}

// Build the standard genres and stations menu
void CreateGenreAndStationMenus()
{
	char szTranslation[64];
	char szParentGenreIndex[12] = "";
	char szParentGenreName[128] = "";
	char szChildGenreIndex[12] = "";
	char szChildGenreName[128] = "";
	char szStationID[16] = "";
	char szStationName[64] = "";
	static int iGenreCount = 0; // Reduce CPU cost by caching
	static int iStationCount = 0; // Same as above
	
	if (g_bUseGenres)
	{
		g_mRadioBaseGenresMenu = new Menu(Handler_SelectGenre);
		
		Format(szTranslation, sizeof(szTranslation), "%T:", "Select Genre", LANG_SERVER);
		g_mRadioBaseGenresMenu.SetTitle(szTranslation);
		
		iGenreCount = g_Genres.iCount;
		
		// Cycle through all the genres and check which are parents
		for (int iParentIndex = 0; iParentIndex < iGenreCount; iParentIndex++)
		{
			if (g_Genres.IsValid(iParentIndex))
			{
				// Is this is a parent genre with children?
				if (g_Genres.GetHasChildren(iParentIndex))
				{
					// Add parent entry to base genres menu
					g_Genres.GetName(iParentIndex, szParentGenreName, sizeof(szParentGenreName));
					IntToString(iParentIndex, szParentGenreIndex, sizeof(szParentGenreIndex));
					g_mRadioBaseGenresMenu.AddItem(szParentGenreIndex, szParentGenreName);
					
					// Initialize parent genre menu
					Handle hMenu = new Menu(Handler_SelectGenre);
					
					g_Genres.SetMenuHandle(iParentIndex, hMenu);
					
					SetMenuTitle(hMenu, szParentGenreName);
					
					// Append the child genres to the parent menus
					for (int iChildIndex = 0; iChildIndex < iGenreCount; iChildIndex++)
					{
						if (g_Genres.IsValid(iChildIndex) && (g_Genres.GetID(iParentIndex) == g_Genres.GetParentID(iChildIndex)))
						{
							IntToString(iChildIndex, szChildGenreIndex, sizeof(szChildGenreIndex));
							g_Genres.GetName(iChildIndex, szChildGenreName, sizeof(szChildGenreName));
							AddMenuItem(hMenu, szChildGenreIndex, szChildGenreName);
						}
					}
					
					SetMenuExitBackButton(hMenu, true);
				}
				else if (g_Genres.GetParentID(iParentIndex) == 0)
				{
					// This must be a base genre menu without children because it has no parent or children
					// Add an entry to the base menu but use the station handler on selection
					Handle hMenu = new Menu(Handler_PlayRadioStation);
					g_Genres.SetMenuHandle(iParentIndex, hMenu);
					g_Genres.GetName(iParentIndex, szParentGenreName, sizeof(szParentGenreName));
					SetMenuTitle(hMenu, szParentGenreName);
					IntToString(iParentIndex, szParentGenreIndex, sizeof(szParentGenreIndex));
					g_mRadioBaseGenresMenu.AddItem(szParentGenreIndex, szParentGenreName);
				}
				else
				{
					// Everything else must be a genre menu with stations
					
					Handle hMenu = new Menu(Handler_PlayRadioStation);
					g_Genres.SetMenuHandle(iParentIndex, hMenu);
					Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
					SetMenuTitle(hMenu, szTranslation);
				}
			}
		}
		
		g_mRadioBaseGenresMenu.ExitBackButton = true;
		
		iStationCount = g_Stations.iCount;
		
		// Now add all the stations to the menus that match the genre ID
		for (int iStationIndex = 0; iStationIndex < iStationCount; iStationIndex++)
		{
			// Make sure the station index is valid and also the station ID
			if (g_Stations.IsValid(iStationIndex) && g_Stations.GetID(iStationIndex) > 0)
			{
				int iParentIndex = g_Genres.GetIndexFromID(g_Stations.GetGenreID(iStationIndex));
				g_Stations.GetName(iStationIndex, szStationName, sizeof(szStationName));
				
				if (!g_Genres.IsValid(iParentIndex) || g_Stations.GetGenreID(iStationIndex) == 0 || !szStationName[0])
				{
					RLogMessage("[SM-RADIO] Error: Station ID \"%d\" with name \"%s\" - Has an invalid genre ID or name", g_Stations.GetID(iStationIndex), szStationName);
				}
				else
				{
					Format(szStationID, sizeof(szStationID), "%d", g_Stations.GetID(iStationIndex));
					AddMenuItem(g_Genres.GetMenuHandle(iParentIndex), szStationID, szStationName);
				}
			}
			else
			{
				RLogMessage("[SM-RADIO] Error: Found an invalid station in CreateGenreAndStationMenus()");
			}
		}
		
		iGenreCount = g_Genres.iCount;
		
		// Add the exit back button to all the station list menus aka ones without children
		for (int iGenreIndex = 0; iGenreIndex < iGenreCount; iGenreIndex++)
		{
			if (g_Genres.IsValid(iGenreIndex) && !g_Genres.GetHasChildren(iGenreIndex))
				SetMenuExitBackButton(g_Genres.GetMenuHandle(iGenreIndex), true);
		}
	}
	else
	{
		// Assume we're not using genres and append stations directly to the base menu
		g_mRadioStationsMenu = new Menu(Handler_PlayRadioStation);
		Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
		g_mRadioStationsMenu.SetTitle(szTranslation);
		
		iStationCount = g_Stations.iCount;
		
		for (int iStationIndex = 0; iStationIndex < iStationCount; iStationIndex++)
		{
			if (g_Stations.IsValid(iStationIndex))
			{
				g_Stations.GetName(iStationIndex, szStationName, sizeof(szStationName));
				Format(szStationID, sizeof(szStationID), "%d", g_Stations.GetID(iStationIndex));
				g_mRadioStationsMenu.AddItem(szStationID, szStationName);
			}
		}
		
		g_mRadioStationsMenu.ExitBackButton = true;
	}
}

// Same as above but using the custom genre/station method maps
void CreateCustomGenreAndStationMenus()
{
	char szTranslation[64];
	char szParentGenreIndex[12] = "";
	char szParentGenreName[128] = "";
	char szChildGenreIndex[12] = "";
	char szChildGenreName[128] = "";
	char szStationID[16] = "";
	char szStationName[64] = "";
	static int iCustGenreCount = 0; // Reduce CPU cost by caching
	static int iCustStationCount = 0; // Same as above
	
	if (g_bCustUseGenres)
	{
		g_mRadioCustomBaseGenresMenu = new Menu(Handler_CustomSelectGenre);
		
		Format(szTranslation, sizeof(szTranslation), "%T:", "Select Genre", LANG_SERVER);
		g_mRadioCustomBaseGenresMenu.SetTitle(szTranslation);
		
		iCustGenreCount = g_CustGenres.iCount;
		
		// Cycle through all the genres and check which are parents
		for (int iParentIndex = 0; iParentIndex < iCustGenreCount; iParentIndex++)
		{
			if (g_CustGenres.IsValid(iParentIndex))
			{
				// Is this is a parent genre with children?
				if (g_CustGenres.GetHasChildren(iParentIndex))
				{
					// Add parent entry to base genres menu
					g_CustGenres.GetName(iParentIndex, szParentGenreName, sizeof(szParentGenreName));
					IntToString(iParentIndex, szParentGenreIndex, sizeof(szParentGenreIndex));
					g_mRadioCustomBaseGenresMenu.AddItem(szParentGenreIndex, szParentGenreName);
					
					// Initialize parent genre menu
					Handle hMenu = new Menu(Handler_CustomSelectGenre);
					
					g_CustGenres.SetMenuHandle(iParentIndex, hMenu);
					
					SetMenuTitle(hMenu, szParentGenreName);
					
					// Append the child genres to the parent menus
					for (int iChildIndex = 0; iChildIndex < iCustGenreCount; iChildIndex++)
					{
						if (g_CustGenres.IsValid(iChildIndex) && (g_CustGenres.GetID(iParentIndex) == g_CustGenres.GetParentID(iChildIndex)))
						{
							IntToString(iChildIndex, szChildGenreIndex, sizeof(szChildGenreIndex));
							g_CustGenres.GetName(iChildIndex, szChildGenreName, sizeof(szChildGenreName));
							AddMenuItem(hMenu, szChildGenreIndex, szChildGenreName);
						}
					}
					
					SetMenuExitBackButton(hMenu, true);
				}
				// This is must be a base genre menu without children because it has no parent
				else if (g_CustGenres.GetParentID(iParentIndex) == 0)
				{
					// Add an entry to the base menu but use the station handler on selection
					Handle hMenu = new Menu(Handler_CustomPlayRadioStation);
					g_CustGenres.SetMenuHandle(iParentIndex, hMenu);
					
					SetMenuTitle(hMenu, szParentGenreName);
					g_CustGenres.GetName(iParentIndex, szParentGenreName, sizeof(szParentGenreName));
					IntToString(iParentIndex, szParentGenreIndex, sizeof(szParentGenreIndex));
					g_mRadioCustomBaseGenresMenu.AddItem(szParentGenreIndex, szParentGenreName);
				}
				// Everything else must be a genre menu with stations
				else
				{
					Handle hMenu = new Menu(Handler_CustomPlayRadioStation);
					g_CustGenres.SetMenuHandle(iParentIndex, hMenu);
					Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
					SetMenuTitle(hMenu, szTranslation);
				}
			}
			else
			{
				RLogMessage("[SM-RADIO] Error : CreateCustomGenreAndStationMenus() Genre at index=%s is invalid");
			}
		}
		
		g_mRadioCustomBaseGenresMenu.ExitBackButton = true;
		
		iCustStationCount = g_CustStations.iCount;
		
		// Now add all the stations to the menus that match the genre ID
		for (int iStationIndex = 0; iStationIndex < iCustStationCount; iStationIndex++)
		{
			// Make sure the station index is valid and also the station ID
			if ((g_CustStations.IsValid(iStationIndex)) && (g_CustStations.GetID(iStationIndex) > 0))
			{
				int iGenreIndex = g_CustGenres.GetIndexFromID(g_CustStations.GetGenreID(iStationIndex));
				g_CustStations.GetName(iStationIndex, szStationName, sizeof(szStationName));
				
				if (!g_CustGenres.IsValid(iGenreIndex) || g_CustStations.GetGenreID(iStationIndex) == 0 || !szStationName[0])
				{
					RLogMessage("[SM-RADIO] Error: Custom Station ID \"%d\" with name \"%s\" - Has an invalid genre ID or name", g_CustStations.GetID(iStationIndex), szStationName);
				}
				else
				{
					Format(szStationID, sizeof(szStationID), "%d", g_CustStations.GetID(iStationIndex));
					AddMenuItem(g_CustGenres.GetMenuHandle(iGenreIndex), szStationID, szStationName);
				}
			}
			else
			{
				RLogMessage("[SM-RADIO] Error: Found an invalid station in CreateCustomGenreAndStationMenus()");
			}
		}
		
		iCustGenreCount = g_CustGenres.iCount;
		
		// Add the exit back button to all the station list menus aka ones without children
		for (int iGenreIndex = 0; iGenreIndex < iCustGenreCount; iGenreIndex++)
		{
			if (g_CustGenres.IsValid(iGenreIndex) && !g_CustGenres.GetHasChildren(iGenreIndex))
				SetMenuExitBackButton(g_CustGenres.GetMenuHandle(iGenreIndex), true);
		}
	}
	else
	{
		// Assume we're not using genres and append stations directly to the base menu
		g_mRadioCustomStationsMenu = new Menu(Handler_CustomPlayRadioStation);
		Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
		g_mRadioCustomStationsMenu.SetTitle(szTranslation);
		
		iCustStationCount = g_CustStations.iCount;
		
		for (int iStationIndex = 0; iStationIndex < iCustStationCount; iStationIndex++)
		{
			if (g_CustStations.IsValid(iStationIndex))
			{
				g_CustStations.GetName(iStationIndex, szStationName, sizeof(szStationName));
				Format(szStationID, sizeof(szStationID), "%d", g_CustStations.GetID(iStationIndex));
				g_mRadioCustomStationsMenu.AddItem(szStationID, szStationName);
			}
		}
		
		g_mRadioCustomStationsMenu.ExitBackButton = true;
	}
}

void CreateRadioVolumeMenu()
{
	char szTranslation[64] = "";
	
	g_mRadioVolumeMenu = new Menu(Handler_VolumeMenu);
	
	Format(szTranslation, sizeof(szTranslation), "%T:", "Volume Menu Title", LANG_SERVER);
	g_mRadioVolumeMenu.SetTitle(szTranslation);
	g_mRadioVolumeMenu.AddItem("0.01", "1%");
	g_mRadioVolumeMenu.AddItem("0.05", "5%");
	g_mRadioVolumeMenu.AddItem("0.10", "10%");
	g_mRadioVolumeMenu.AddItem("0.20", "20%");
	g_mRadioVolumeMenu.AddItem("0.40", "40%");
	g_mRadioVolumeMenu.AddItem("0.60", "60%");
	g_mRadioVolumeMenu.AddItem("0.80", "80%");
	g_mRadioVolumeMenu.AddItem("1.0", "100%");
	g_mRadioVolumeMenu.ExitBackButton = true;
}
